'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _size2 = require('lodash/size');

var _size3 = _interopRequireDefault(_size2);

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _reduce2 = require('lodash/reduce');

var _reduce3 = _interopRequireDefault(_reduce2);

exports.default = combineReducers;

var _immutable = require('immutable');

var _mindfrontReduxUtils = require('mindfront-redux-utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function combineReducers(reducers) {
  var createInitialState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _immutable.Map;

  var actionHandlers = {};
  var otherReducers = {};
  var initialStateObj = {};

  // regroup reducers that have actionHandlers:
  //   from: prop name -> action type -> reducer
  //     to: action type -> prop name -> reducer
  // put reducers without actionHandlers into a prop name -> reducer map

  (0, _forEach3.default)(reducers, function (reducer, key) {
    initialStateObj[key] = reducer(undefined, {});
    if (reducer.actionHandlers) {
      (0, _forEach3.default)(reducer.actionHandlers, function (actionHandler, type) {
        (actionHandlers[type] || (actionHandlers[type] = {}))[key] = actionHandler;
      });
    } else otherReducers[key] = reducer;
  });
  var initialState = createInitialState(initialStateObj);

  // now we can create a more efficient reducer for each action type
  // and one more efficient reducer for the otherReducers

  // creates an efficient reducer from a prop name -> reducer map
  var combineBase = function combineBase(reducers) {
    var result = void 0;
    if ((0, _size3.default)(reducers) > 1) {
      // we have to update multiple keys; use state.withMutations
      result = function result() {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments[1];
        return state.withMutations(function (mutableState) {
          return (0, _reduce3.default)(reducers, function (nextState, reducer, key) {
            return nextState.update(key, function (value) {
              return reducer(value, action);
            });
          }, mutableState);
        });
      };
    } else {
      // we only have to update one key; use state.update
      var key = Object.keys(reducers)[0];
      var reducer = reducers[key];
      result = function result() {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments[1];
        return state.update(key, function (value) {
          return reducer(value, action);
        });
      };
    }
    result.domainHandlers = reducers;
    return result;
  };

  var actionHandlerReducer = (0, _size3.default)(actionHandlers) ? (0, _mindfrontReduxUtils.createReducer)(initialState, (0, _mapValues3.default)(actionHandlers, combineBase)) : undefined;

  var otherReducer = (0, _size3.default)(otherReducers) ? combineBase(otherReducers) : undefined;

  if (actionHandlerReducer && otherReducer) return (0, _mindfrontReduxUtils.composeReducers)(actionHandlerReducer, otherReducer);
  if (actionHandlerReducer) return actionHandlerReducer;
  if (otherReducer) return otherReducer;
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    return state;
  };
}